
pstar = pload("img/star.png")
pring = pload("img/ring.png")

metaStar = {}
function metaStar:make(vx, vy, vdx, vdy, vs)
	local o = {}
	setmetatable(o, metaStar)
	self.__index = self
	o.name = "star"
	o.rect = metaRect.make(vx - 4, vy - 4, 8, 8)
	o.offRect = metaRect.make(vx - 2, vy - 2, 4, 4)
	o.lFade = {}
	o.offX = 0
	o.offY = 0
	o.iso = false
	o.canOffLeft = true
	o.canOffRight = true
	o.canOffUp = true
	o.canOffDown = true
	o.canScoreLeft = true
	o.canScoreRight = true
	o.canScoreUp = true
	o.canScoreDown = true
	o.dx = vdx
	o.dy = vdy
	o.s = vs
	o.r = 0
	o.fade = 0
	return o
end

function metaStar:run(key, dt)
	self:limitOff()
	self:loopBlocks()
	
	self:makeFade(dt)
	self:runFade(dt)
	self.r = self.r + (5 * dt)

	if self.dx then
		if self.dy then
			self.rect:moveX(-self.s, dt)
			self.rect:moveY(-self.s, dt)
		else
			self.rect:moveX(-self.s, dt)	
			self.rect:moveY(self.s, dt)
		end
	else
		if self.dy then
			self.rect:moveX(self.s, dt)
			self.rect:moveY(-self.s, dt)
		else
			self.rect:moveX(self.s, dt)
			self.rect:moveY(self.s, dt)
		end
	end
	self:runOff(key, dt)
	self:retractOff(dt)
	
	self.canOffLeft = true
	self.canOffRight = true
	self.canOffUp = true
	self.canOffDown = true
end

function metaStar:runPush(dt)
	local aDown = false
	local dDown = false
	local wDown = false
	local sDown = false
	
	local pushSpeed = 75
	
	if(love.keyboard.isDown("a")) then aDown = true end
	if(love.keyboard.isDown("d")) then dDown = true end
	if(love.keyboard.isDown("w")) then wDown = true end
	if(love.keyboard.isDown("s")) then sDown = true end
	
	--//--
	
	if(aDown == true) then
		self.rect:moveX(-pushSpeed, dt)
	elseif(dDown == true) then
		self.rect:moveX(pushSpeed, dt)
	elseif(wDown == true) then
		self.rect:moveY(-pushSpeed, dt)
	elseif(sDown == true) then
		self.rect:moveY(pushSpeed, dt)
	end
	
end

function metaStar:runOff(key, dt)
	local os = 250
	
	self.iso = false
	if key.w and self.canOffUp then
		self.iso = true
		self.offY = self.offY - (os * dt)
	elseif key.s and self.canOffDown then
		self.offY = self.offY + (os * dt)
		self.iso = true
	end
	if key.a and self.canOffLeft then
		self.offX = self.offX - (os * dt)
		self.iso = true
	elseif key.d and self.canOffRight then
		self.offX = self.offX + (os * dt)
		self.iso = true
	end
	
	if self.offX > -4 and self.offX < 4 then self.offX = 0 end
	if self.offY > -4 and self.offY < 4  then self.offY = 0 end
	
	self.offRect.x = self.rect.x + self.offX
	self.offRect.y = self.rect.y + self.offY
end

function metaStar:retractOff(dt)
	local ss = 500
	if self.iso == false then
		if(self.offX > 0.5) then
			self.offX = self.offX - (ss * dt)
		elseif(self.offX < -0.5) then
			self.offX = self.offX + (ss * dt)
		end
		if(self.offY > 0.5) then
			self.offY = self.offY - (ss * dt)
		elseif(self.offY < -0.5) then
			self.offY = self.offY + (ss * dt)
		end
	end
end

function metaStar:limitOff()
	if(self.offRect.x < 48) then self.offRect.x = 48 end
	if(self.offRect.x > 800 - 48) then self.offRect.x = 800 - 48 end
	if(self.offRect.y < 48) then self.offRect.y = 48 end
	if(self.offRect.y > 600 - 48) then self.offRect.y = 600 - 48 end
	
	if((self.rect.x + self.offX) > (800 - 32)) then self.offX = ((800 - 32) - self.rect.x) end
	if((self.rect.x + self.offX) < 32) then self.offX = (32 - self.rect.x) end
	if((self.rect.y + self.offY) > (600 - 32)) then self.offY = ((600 - 32) - self.rect.y) end
	if((self.rect.y + self.offY) < 32) then self.offY = (32 - self.rect.y) end
end

function metaStar:makeFade(dt)
	self.fade = self.fade + ((self.s * 2) * dt)
	if(self.fade > 15) then
		nFade = {x = self.offRect.x, y = self.offRect.y, opa = 120, isd = false}
		table.insert(self.lFade, nFade)
		self.fade = 0
	end
end

function metaStar:runFade(dt)
	for i, v in ipairs(self.lFade) do
		v.opa = v.opa - ((self.s * 2) * dt)
		if v.opa < 10 then table.remove(self.lFade, i) end
	end
end

function metaStar:drawFade()
	grab = nil
	last = nil
	for i, v in ipairs(self.lFade) do
		if(grab ~= nil) then
			theColor:setWithAlpha(v.opa)
			love.graphics.line(grab.x, grab.y, v.x, v.y)
		 	grab = v
			last = v
		else
			grab = v
		end
	end
	if last ~= nil then love.graphics.line(last.x, last.y, self.offRect.x, self.offRect.y) end
end

function metaStar:hitBar(l, lrud)
	if(self.rect:doHit(l.sq)) then
		if(lrud == Left) then
			self.dx = false
		elseif(lrud == Right) then
			self.dx = true
		elseif(lrud == Up) then
			self.dy = false
		elseif(lrud == Down) then
			self.dy = true
		end
		return true
	end
end

function metaStar:hitBoy(vrect, lrud)
	
	local hit = false
	if(self.rect:doHit(vrect) == true) then
		hit = true
	end
	
	if(hit == true and lrud == glol) then
		self.dx = false
		return true
	end
	if(hit == true and lrud == glor) then
		self.dx = true
		return true
	end
	if(hit == true and lrud == glou) then
		self.dy = false
		return true
	end
	if(hit == true and lrud == glod) then
		self.dy = true
		return true
	end
	return false
end

function metaStar:pokeBoy(vsq, l, lrud)
	
	if(vsq:doHit(l.psq) == true) then
		if(lrud == Left) then
			self.dx = false
			return true
		end
		if(lrud == Right) then
			self.dx = true
			return true
		end
		if(lrud == Up) then
			self.dy = false
			return true
		end
		if(lrud == Down) then
			self.dy = true
			return true
		end
	end
	
	return false
end

function metaStar:minusLife(vSide)
	if vSide == Left then lBarLeft.life = lBarLeft.life - 1 end
	if vSide == Right then lBarRight.life = lBarRight.life - 1 end
	if vSide == Up then lBarUp.life = lBarUp.life - 1 end
	if vSide == Down then lBarDown.life = lBarDown.life - 1 end
end

function metaStar:loopBlocks()
	
	for i, v in ipairs(lBlock) do
		if self.rect:doHit(v.rect) then
			local grabSide = self.rect:sideHit(v.rect)
			
			--normal block
			if v.type == 0 then
				if grabSide == Left then
					self.dx = true
				end
				if grabSide == Right then
					self.dx = false
				end
				if grabSide == Up then
					self.dy = true
				end
				if grabSide == Down then
					self.dy = false
				end
			end
			
			if v.type == 3 then
				v.noUse = true
			end
		end
	end
	
	for i, v in ipairs(lBlock) do
		if self.offRect:doHit(v.rect) then
			local grabSide = self.offRect:sideHit(v.rect)
			
			--base block
			if v.type == 0 then
				if grabSide == Left and self.rect.x < v.rect.x + v.rect.width then
					self.dx = true
					v.pokeLeft = 255
					self.canOffRight = false
				end
				if grabSide == Right and self.rect.x > v.rect.x + v.rect.width then
					self.dx = false
					v.pokeRight = 255
					self.canOffLeft = false
				end
				if grabSide == Up and self.rect.y < v.rect.y + v.rect.height then
					self.dy = true
					v.pokeUp = 255
					self.canOffDown = false
				end
				if grabSide == Down and self.rect.y > v.rect.y + v.rect.height then
					self.dy = false
					v.pokeDown = 255
					self.canOffUp = false
				end
			
			--starblock
			elseif v.type == 1 then
				lGame.score = lGame.score + 20
				v.isd = true
			
			elseif v.type == 3 then
			if grabSide == Left and self.rect.x < v.rect.x + v.rect.width then
				self.dx = true
				v.pokeLeft = 255
				self.canOffRight = false
			end
			if grabSide == Right and self.rect.x > v.rect.x + v.rect.width then
				self.dx = false
				v.pokeRight = 255
				self.canOffLeft = false
			end
			if grabSide == Up and self.rect.y < v.rect.y + v.rect.height then
				self.dy = true
				v.pokeUp = 255
				self.canOffDown = false
			end
			if grabSide == Down and self.rect.y > v.rect.y + v.rect.height then
				self.dy = false
				v.pokeDown = 255
				self.canOffUp = false
			end
		end
		end

	end
end

function metaStar:draw()
	self:drawFade()
	theColor:set()
	love.graphics.line(self.rect.x - 10, self.rect.y, self.offRect.x, self.offRect.y)
	love.graphics.line(self.rect.x + 10, self.rect.y, self.offRect.x, self.offRect.y)
	love.graphics.line(self.rect.x, self.rect.y - 10, self.offRect.x, self.offRect.y)
	love.graphics.line(self.rect.x, self.rect.y + 10, self.offRect.x, self.offRect.y)
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(pring, self.rect.x, self.rect.y, self.r, 1, 1, 16, 16)
	love.graphics.draw(pstar, self.offRect.x, self.offRect.y, self.r, 1, 1, 12, 12)
end
